package pong;

import java.awt.Graphics;
import java.awt.Point;

/**
 * Classe que representa a bola do jogo. Esta classe dispe de mtodos para mover e redezenhar
 * a bola. <p> ATENO: No  necessrio criar uma instncia desta classe directamente. Isto dever ser efectuado
 * pela API fornecida.</p>
 * 
 * @author  nuno
 */
public class Ball {

	private BallState ballState;

	private int right_out = 390;
	private int left_out = 10;
	private int down_out = 290;
	private int up_out = 10;

	final int radius = 10;
	
	private Paddle[] paddles;

	/**
	 * Construtor da classe.
	 * @param x_field posio xx do campo.
	 * @param y_field posio yy do campo.
	 * @param w largura do campo.
	 * @param h altura do campo.
	 * @param paddles conjunto (ou array) de Paddles.
	 */
	public Ball(int x_field, int y_field, int w, int h, Paddle[] paddles) {
		init(x_field,y_field,w,h,paddles);
		ballState = new BallState(200,300,8,6);
	}

	/**
	 * Construtor da classe.
	 * @param initialPos posio inicial da bola.
	 * @param x_field posio xx do campo.
	 * @param y_field posio yy do campo.
	 * @param w largura do campo.
	 * @param h altura do campo.
	 * @param paddles conjunto (ou array) de Paddles.
	 */
	public Ball(Point initialPos, int x_field, int y_field, int w, int h, Paddle[] paddles) {
		init(x_field,y_field,w,h,paddles);
		ballState = new BallState(initialPos.x,initialPos.y,8,6);
	}

	private void init(int x_field, int y_field, int w, int h, Paddle[] paddles){
		this.paddles = paddles;
		left_out = x_field;
		up_out = y_field;
		right_out = w - radius;
		down_out = h - radius;		
	}

	/**
	 * Move a bola para a posio seguinte e redezenha-a no ecran.
	 * @param g grfico do componente onde a bola dever ser redezenhada.
	 * @throws OutOfFieldException se a bola bateu numa parede.
	 */
	public void move(Graphics g) throws OutOfFieldException {
		clear(g);
		ballState.incPosition();
		paint(g);
		if(!isBallOnPaddles()){
			int player = -1;
			if((player = isBallOut()) != -1)
			throw new OutOfFieldException("Goooal!!!", player);
		}
	}

	/**
	 * Move a bola para a posio dada e redezenha-a no ecran.
	 * @param newState novo estado da bola.
	 * @param g grfico do componente onde a bola dever ser redezenhada.
	 * @throws OutOfFieldException se a bola bateu numa parede.
	 */
	public void move(BallState newState, Graphics g){
		clear(g);
		ballState = newState;
		paint(g);
	}

	/**
	 * Devolve o actual estado da bola.
	 * @return o actual estado da bola.
	 */
	protected BallState getBallState() {
		return ballState;
	}

	/**
	 * Actualiza o estado da bola.
	 * @param ballState o novo estado da bola.
	 */
	protected void setBallState(BallState ballState) {
		this.ballState = ballState;
	}
	
	private boolean isBallOnPaddles(){
		for (int i=0; i<paddles.length; i++)
			if( paddles[i].isBallOnPaddle(ballState) )
				return true;
		return false;
	}

	private int isBallOut() {
		if (ballState.getXSpeed() < 0) {
			if (ballState.getXPosition() < left_out) {
				ballState.setXSpeed(-ballState.getXSpeed());
				return Paddle.LEFT;
			}
		} else if (ballState.getXSpeed() > 0) {
			if (ballState.getXPosition() > right_out) {
				ballState.setXSpeed(-ballState.getXSpeed());
				return Paddle.RIGHT;
			}
		}
		if (ballState.getYSpeed() < 0) {
			if (ballState.getYPosition() < up_out) {
				ballState.setYSpeed(-ballState.getYSpeed());
				return Paddle.TOP;
			}
		} else if (ballState.getYSpeed() > 0) {
			if (ballState.getYPosition() > down_out) {
				ballState.setYSpeed(-ballState.getYSpeed());
				return Paddle.BOTTOM;
			}
		}
		return -1;
	}

	private void clear(Graphics g) {
		g.clearRect(ballState.getXPosition() - radius, ballState.getYPosition() - radius, 2 * radius, 2 * radius);
	}

	/**
	 * Redezenha a bola, tendo em conta o seu estado actual.
	 * @param g Grficos do componente onde dever ser redezenhada a bola.
	 */
	protected void paint(Graphics g) {
		g.fillOval(ballState.getXPosition() - radius, ballState.getYPosition() - radius, 2 * radius, 2 * radius);
		for(int i=0; i<paddles.length; i++)
			paddles[i].paint(g);
	}

}
